<?php


namespace Schaffen\Core\Movement\Managers;


use Carbon\Carbon;
use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Schaffen\Core\Movement\Repositories\SalesOrderItemRepository;
use Schaffen\Core\Movement\Repositories\SalesOrderPaymentFormRepository;
use Schaffen\Core\Movement\Repositories\SalesOrderPaymentFormInstallmentsRepository;
use Schaffen\Core\Movement\Repositories\SalesOrderRepository;
use Schaffen\Core\Movement\Services\SalesOrderValidateService;

class SalesOrderManager extends ManipulationManager
{

    protected $validation = [
        self::CREATE => [

        ],
        self::EDIT => [

        ],
        self::CREATE_MESSAGE => [

            "pes_codigo.required" => 'O código da pessoa é obrigatório',
            "pev_data_movel.required" => 'A data do pedido é obrigatória',
        ],
        self::EDIT_MESSAGE => [

        ],
    ];
    protected $repository;
    public $error = [];

    public function __construct(
        SalesOrderRepository            $repository,
        SalesOrderItemRepository        $sales_item_repository,
        SalesOrderPaymentFormRepository $order_payment_form_repository,
        SalesOrderPaymentFormInstallmentsRepository $order_payment_form_installments_repository,
        SalesOrderValidateService       $sales_order_validate_service
    )
    {
        $this->repository = $repository;
        $this->sales_item_repository = $sales_item_repository;
        $this->order_payment_form_repository = $order_payment_form_repository;
        $this->order_payment_form_installments_repository = $order_payment_form_installments_repository;
        $this->sales_order_validate_service = $sales_order_validate_service;
    }

    public function ultimosPedidos($values)
    {
        $per_page = $values['per_page'] ?? 10;
        return $this->repository->setOrder('pev_codigo', 'desc')
            ->list([
                'pev_fk_pes_codigo_vdr' => $values['person']],
                ['itens_do_pedido', 'forma_pagamento_pedido', 'cliente']
                , $per_page);
    }

    public function addPedido($values)
    {
        $response = $this->sales_order_validate_service->validar_pedido($values);

        if ($response['status_code'] == 500) {
            return $response;
        } else {
            $id = $values['pev_codigo'] ?? null;
            $pedido = [];
            $pedido['pev_fk_pes_codigo'] = $values['pes_codigo'];
            $pedido['pev_fk_pes_codigo_vdr'] = $values['seller'];
            $pedido['pev_data_movel'] = $values['pev_data_movel'];
            $novo_pedido = $this->repository->storeOrUpdate($pedido, $id);
            $response['pedido'] = $novo_pedido;

            $itens_do_pedido['pev_codigo'] = $novo_pedido['pev_codigo'];
            $itens_do_pedido['itens'] = $values['itens'];
            $response['itens'] = $this->sales_item_repository->storeOrUpdate($itens_do_pedido);

            $response['forma_pagamento'] = [];
            foreach ($values['forma_pagamento'] as $value){
                $value['pvfp_fk_pev_codigo'] = $novo_pedido['pev_codigo'];
                $nova_fp = $this->order_payment_form_repository->storeOrUpdate($value);
                if($value['pvfp_numero_parcelas'] != null){
                    $parcelas = [];
                    $parcelas['parcelas'] = $value['parcelas'];
                    $parcelas['pvfpp_fk_pvfp_codigo'] = $nova_fp['pvfp_codigo'];
                    $parcela_ret = $this->order_payment_form_installments_repository->storeOrUpdate($parcelas);
                    $nova_fp['parcelas'] = $parcela_ret;
                }
                $response['forma_pagamento'][] = $nova_fp;
            }
        }

        return $response;
    }

    public function cancelarPedido($values)
    {
        $pedido_venda = $this->repository->first(['pev_codigo' =>$values['pev_codigo']]);
        if(!$pedido_venda->pev_cancelado){
            $pedido_venda = $this->repository->storeOrUpdate(['pev_cancelado' => Carbon::now()], $values['pev_codigo']);
        }
        return $pedido_venda;
    }


}
