<?php


namespace Schaffen\Core\Person\Managers;


use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Schaffen\Core\Person\Models\Address;
use Schaffen\Core\Person\Models\Person;
use Schaffen\Core\Person\Repositories\ContactRepository;
use Schaffen\Core\Person\Repositories\PersonBranchRepository;
use Schaffen\Core\Person\Repositories\PersonModalityRepository;
use Schaffen\Core\Person\Repositories\PersonRepository;
use Schaffen\Core\User\Repositories\UserRepository;
use Schaffen\Financial\BillingRoute\Repositories\ClientRouteRepository;
use Schaffen\Core\Person\Repositories\AddressRepository;
use Schaffen\Core\Person\Services\ClientServices;


class ClientManager extends ManipulationManager
{
    protected $validation = [
        self::CREATE => [
            'pes_nome' => 'required',
            'pes_identificador' => 'required'
        ],
        self::EDIT => [],

        self::CREATE_MESSAGE => [

            "pes_nome.required" => 'O código da pessoa é obrigatório.',
            "pes_identificador.required" => 'O identificador da pessoa é obrigatório.',

        ],
        self::EDIT_MESSAGE => [

        ],
    ];

    /**
     * @var PersonRepository
     */
    protected $repository;


    public function __construct(
        PersonRepository  $repository,
        PersonManager     $person_manager,
        UserRepository    $user_repository,
        AddressRepository $address_repository,
        AddressManager    $address_manager,
        ContactRepository $contact_repository,
        PersonModalityRepository $person_modality_repository,
        ClientRouteRepository $client_route_repository,
        ClientServices $client_services,
        PersonBranchRepository $person_branch_repository


    )
    {
        $this->repository = $repository;
        $this->user_repository = $user_repository;
        $this->address_repository = $address_repository;
        $this->address_manager = $address_manager;
        $this->contact_repository = $contact_repository;
        $this->person_manager = $person_manager;
        $this->person_modality_repository = $person_modality_repository;
        $this->client_services = $client_services;
        $this->client_route_repository = $client_route_repository;
        $this->person_branch_repository = $person_branch_repository;
    }

    /**
     * @param $values
     * @param int|null $id
     * @param array $relations
     * @return Person
     * @throws \Throwable
     */
    public function storeOrUpdate($values, int $id = null, $relations = [])
    {
        
        $response = $this->client_services->validateClient($values);
        
        if($response['status_code'] == 500){
            return $response;
        }
        
        $id = $values['pes_codigo'] ?? null;
        $add_peson_modality = false;

        $values['pes_user_insert'] = $this->client_services->getUser($values);
        
        $values['pes_identificador'] = $this->client_services->limpaCPF_CNPJ($values['pes_identificador']);

        $person = $this->repository->first(['pes_identificador' => $values['pes_identificador']], ['address_default', 'contact_default', 'rota_cliente', 'branch']);

        $person ?? $add_peson_modality = true;

        $pes_codigo = $person->pes_codigo != null ? $person->pes_codigo : $id;
        $roc_fk_rot_codigo = $this->client_services->getRotaVendedor($values);
        $roc_codigo = $person->rota_cliente->roc_codigo ?? null;
        $pee_codigo = $person->address_default->pee_codigo ?? null;
        $peco_codigo = $person->contact_default->peco_codigo ?? null;
        $pef_codigo = $person->branch->pef_codigo ?? null;

        $person = $this->repository->storeOrUpdate($values, $pes_codigo);
        if($person){
            
            $response['status_code'] = 200;
            if($pes_codigo){
                $response['messages'][] = "Cliente editado.";
            }else{
                $response['messages'][] = "Cliente adicionado.";
            }
        }else{
            $response['status_code'] = 500;
            $response['messages'][] = "Erro ao cadastrar cliente.";
            return $response;
        }

        // store person modality : "CLIENTE"
        if($add_peson_modality){
            $values['person_modality'] = $this->client_services->getPersonModality($values, $person->pes_codigo);
            $person_modality = $this->person_modality_repository->storeOrUpdate($values['person_modality']);
        }

        // store route client
        $values['client_route'] = $this->client_services->setClientRoute($values,$roc_fk_rot_codigo, $person->pes_codigo);
        $client_route = $this->client_route_repository->storeOrUpdate($values['client_route'],$roc_codigo);

        // store address
        $values["address"] = $this->client_services->setAddress($values,$person['pes_codigo']);
        $address = $this->address_repository->storeOrUpdate($values["address"], $pee_codigo);

        // store contact
        $values["contact"] = $this->client_services->setContact($values,$person['pes_codigo']);
        $contact = $this->contact_repository->storeOrUpdate($values["contact"], $peco_codigo);

        $values['person_branch'] = $this->client_services->setPersonBranch($values, $person->pes_codigo);
        $person_branch = $this->person_branch_repository->storeOrUpdate($values['person_branch'],$pef_codigo);


        $response['client'] = $this->repository->first(['pes_identificador' => $values['pes_identificador']], ['address_default', 'contact_default', 'rota_cliente']);
        return $response;


    }




}
